/*	File			DNS_Resolver.java
	Purpose			Demonstration of the use of the gethostbyname DS resolver
	Author			Richard Anthony	(ar26@gre.ac.uk)
	Date			March 2015
*/

package DNS_Resolver;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.*;
import javax.swing.ScrollPaneConstants;
import javax.swing.JOptionPane;
import javax.swing.*;
import sun.net.spi.nameservice.dns.DNSNameService;
    
public class DNS_Resolver extends javax.swing.JFrame implements ActionListener
{
    DefaultListModel m_listModel_LocalAddressList; // For use with JList
    DefaultListModel m_listModel_ResolvedAddressList; // For use with JList

    public DNS_Resolver()
    {
        m_listModel_LocalAddressList = new DefaultListModel(); // For use with JList
        m_listModel_ResolvedAddressList = new DefaultListModel(); // For use with JList
        initComponents();
        InitialiseControls();
    }

    private void InitialiseControls()    
    {    
        jTextField_MachineName.setEnabled(false);
        jTextField_MachineName.setText("");
        jList_LocalIPAddressList.setEnabled(false);
        jList_LocalIPAddressList.removeAll();
        jTextField_DomainName.setEnabled(true);
        jList_ResolvedIPAddressList.setEnabled(false);
        jList_ResolvedIPAddressList.removeAll();
        jButton_Resolve.setEnabled(true);
        jButton_Done.setEnabled(true); 
        GetLocalIPAddress();
    }

    private void GetLocalIPAddress()
    {   
        try
        {
            InetAddress LocalAddress = InetAddress.getLocalHost();
            String sHostAddress = LocalAddress.toString(); // Expected format: "Hostname/192.168.100.5"
            int iStartOfDottedIPSubstring = sHostAddress.indexOf('/');
            if(-1 != iStartOfDottedIPSubstring)
            {
                String sHostName = sHostAddress.substring(0, iStartOfDottedIPSubstring);
                InetAddress[] LocalAddressArray = InetAddress.getAllByName(sHostName);
                for(int iIndex = 0; iIndex < LocalAddressArray.length; iIndex++)
                {
                    String sIPAddress = ByteArray_To_IPAddessString(LocalAddressArray[iIndex].getAddress());
                    if(!sIPAddress.isEmpty())
                    {   // The address is an IPv4 or IPv6 address
                        String sCanonicalHostName = LocalAddressArray[iIndex].getCanonicalHostName();
                        jTextField_MachineName.setText(sCanonicalHostName);
                        m_listModel_LocalAddressList.addElement(sIPAddress);
                    }
                }
            }
        }
        catch (Exception Ex)
        {
            JOptionPane.showMessageDialog(null,Ex.toString(),"DNS_Resolver",JOptionPane.PLAIN_MESSAGE);
            System.exit(0);
        }
    }
    
    private void Do_Resolve()
    {
        jButton_Resolve.setEnabled(false);
        jTextField_DomainName.setEnabled(false);
        m_listModel_ResolvedAddressList.clear(); // Remove any previous results
        String sHost = jTextField_DomainName.getText(); // Get the textual name provided by the user
        if (sHost.isEmpty())
        {
            m_listModel_ResolvedAddressList.addElement("Host / domain name is blank");
            jButton_Resolve.setEnabled(true);
            jTextField_DomainName.setEnabled(true);
            return;
        }
        try
        {
            DNSNameService DNS = new DNSNameService();
            InetAddress[] IPAddressArray = DNS.lookupAllHostAddr(sHost);

            for(int iIndex = 0; iIndex < IPAddressArray.length; iIndex++)
            {
                String sIPAddress = ByteArray_To_IPAddessString(IPAddressArray[iIndex].getAddress());
                if(!sIPAddress.isEmpty())
                {   // The address is an IPv4 or IPv6 address
                    m_listModel_ResolvedAddressList.addElement(sIPAddress);
                }
            }
        }
        catch (Exception Ex)
        {
            m_listModel_ResolvedAddressList.addElement(Ex.toString());
        }
        jButton_Resolve.setEnabled(true);
        jTextField_DomainName.setEnabled(true);
    }
    
    private String ByteArray_To_IPAddessString(byte[] bVal)
    {
        String sAddress = "";
        int iAddressComponent = 0;
        if(4 == bVal.length)
        {   // Generate dotted decimal IPv4 address string in format ddd.ddd.ddd.ddd
            for(int iIndex = 0; iIndex < bVal.length-1; iIndex++)
            {
                iAddressComponent = bVal[iIndex] & 0xFF;
                sAddress += Integer.toString(iAddressComponent) + ".";
            }
            iAddressComponent = bVal[bVal.length-1] & 0xFF;
            sAddress += Integer.toString(iAddressComponent); // Add last component (which is not followed by '.')
            return sAddress;
        }

        if(16 == bVal.length)
        {   // Generate : separated hexadecimal IPv6 address string in format hhhh.hhhh.hhhh.hhhh.hhhh.hhhh.hhhh.hhhh
            for(int iIndex = 0; iIndex < bVal.length-2; iIndex+=2)
            {   // Note that the loop index is increased by 2 each iteration
                iAddressComponent = (256*(bVal[iIndex] & 0xFF)) + (bVal[iIndex+1] & 0xFF);
                sAddress += Integer.toString(iAddressComponent,16) + ":";
            }
            iAddressComponent = (256*(bVal[bVal.length-2] & 0xFF)) + (bVal[bVal.length-1] & 0xFF);
            sAddress += Integer.toString(iAddressComponent,16); // Add last component (which is not followed by '.')
            return sAddress;
        }
        // Return an empty string if the address is not an IPv4 or IPv6 address
        return sAddress;
    }
    
    public static void main(String args[]) throws Exception
    {
        // Initialise the GUI libraries
        try 
        {
            javax.swing.UIManager.LookAndFeelInfo[] installedLookAndFeels=javax.swing.UIManager.getInstalledLookAndFeels();
            for (int idx=0; idx<installedLookAndFeels.length; idx++)
            {
                if ("Nimbus".equals(installedLookAndFeels[idx].getName())) {
                    javax.swing.UIManager.setLookAndFeel(installedLookAndFeels[idx].getClassName());
                    break;
                }
            }
        } 
        catch (Exception Ex)
        {
            System.exit(0);
        }

        java.awt.EventQueue.invokeAndWait(new Runnable() // Create and display the GUI form
        { 
            public void run() 
            { 
                new DNS_Resolver().setVisible(true);
            }
        } );
    }    

    public void actionPerformed(ActionEvent e)
    {
        if(jButton_Done == e.getSource())
        {
            System.exit(0);
        }
        if(jButton_Resolve == e.getSource())
        {
            Do_Resolve();
        }
    }
    
    private JPanel jPanel_LocalDetails;
    private JLabel jLabel_MachineName;
    private JTextField jTextField_MachineName;
    private JLabel jLabel_LocalIPAddressList;
    private JList jList_LocalIPAddressList;
    private JScrollPane JScrollPane_LocalIPAddressList;
    private JPanel jPanel_DomainName;
    private JTextField jTextField_DomainName;
    private JPanel jPanel_ResolvedDetails;
    private JLabel jLabel_ResolvedIPAddressList;
    private JList jList_ResolvedIPAddressList;
    private JScrollPane JScrollPane_ResolvedIPAddressList;
    private JButton jButton_Resolve;
    private JButton jButton_Done;

    private void initComponents()
    {
        jLabel_MachineName = new JLabel();
        jLabel_MachineName.setText("Machine name");
        jTextField_MachineName = new JTextField();
        jTextField_MachineName.setMaximumSize(new Dimension(330, 4));
        jTextField_MachineName.setHorizontalAlignment(JTextField.CENTER);
        jLabel_LocalIPAddressList = new JLabel();
        jLabel_LocalIPAddressList.setText("IP Address list");
        jList_LocalIPAddressList = new JList(m_listModel_LocalAddressList);
        jList_LocalIPAddressList.setMaximumSize(new Dimension(330, 250));
        jList_LocalIPAddressList.setSelectedIndex(0);
        jList_LocalIPAddressList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        JScrollPane_LocalIPAddressList = new javax.swing.JScrollPane(jList_LocalIPAddressList,
                ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, 
                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        jPanel_LocalDetails = new JPanel();
        jPanel_LocalDetails.setPreferredSize(new Dimension(360, 300));
        jPanel_LocalDetails.setBorder(BorderFactory.createTitledBorder("Local computer address details"));
 
        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel_LocalDetails);
        jPanel_LocalDetails.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
                .add(jPanel1Layout.createSequentialGroup()
                    .add(jLabel_MachineName)
                    .addContainerGap(20, 20)
                    .add(jTextField_MachineName))
                .add(jPanel1Layout.createSequentialGroup()
                    .add(jLabel_LocalIPAddressList)
                    .addContainerGap(20, 20)
                    .add(JScrollPane_LocalIPAddressList)));
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
                .add(jPanel1Layout.createSequentialGroup()
                    .add(jPanel1Layout.createParallelGroup()
                        .add(jLabel_MachineName)
                        .add(jTextField_MachineName))            
                    .addContainerGap(20, 20)
                    .add(jPanel1Layout.createParallelGroup()
                        .add(jLabel_LocalIPAddressList)
                        .add(JScrollPane_LocalIPAddressList))));
        
        jTextField_DomainName = new JTextField();
        jTextField_DomainName.setMaximumSize(new Dimension(360, 4));
        jTextField_DomainName.setHorizontalAlignment(JTextField.CENTER);
        jPanel_DomainName = new JPanel();
        jPanel_DomainName.setPreferredSize(new Dimension(360, 100));
        jPanel_DomainName.setBorder(BorderFactory.createTitledBorder("Domain Name"));

        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(jPanel_DomainName);
        jPanel_DomainName.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
                .add(jPanel2Layout.createSequentialGroup()
                    .add(jTextField_DomainName)));
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
                .add(jPanel2Layout.createSequentialGroup()
                    .add(jTextField_DomainName)));            
                
        jLabel_ResolvedIPAddressList = new JLabel();
        jLabel_ResolvedIPAddressList.setText("IP Address list");
        jList_ResolvedIPAddressList = new JList(m_listModel_ResolvedAddressList);
        jList_ResolvedIPAddressList.setMaximumSize(new Dimension(330, 250));
        jList_ResolvedIPAddressList.setSelectedIndex(0);
        jList_ResolvedIPAddressList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        JScrollPane_ResolvedIPAddressList = new javax.swing.JScrollPane(jList_ResolvedIPAddressList,
                ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, 
                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        jButton_Resolve = new JButton();
        jButton_Resolve.setText("Resolve");
        jButton_Resolve.setMaximumSize(new Dimension(100, 4));
        jButton_Resolve.addActionListener(this);
        jButton_Done = new JButton();
        jButton_Done.setText("Done");
        jButton_Done.setMaximumSize(new Dimension(100, 4));
        jButton_Done.addActionListener(this);
        
        jPanel_ResolvedDetails = new JPanel();
        jPanel_ResolvedDetails.setPreferredSize(new Dimension(360, 300));
        jPanel_ResolvedDetails.setBorder(BorderFactory.createTitledBorder("Resolved address details"));

        org.jdesktop.layout.GroupLayout jPanel3Layout = new org.jdesktop.layout.GroupLayout(jPanel_ResolvedDetails);
        jPanel_ResolvedDetails.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                .add(jPanel3Layout.createSequentialGroup()
                    .add(jPanel3Layout.createParallelGroup()
                        .add(jLabel_ResolvedIPAddressList)
                        .add(jButton_Resolve)
                        .add(jButton_Done))
                    .addContainerGap(20, 20)
                    .add(jPanel3Layout.createParallelGroup()
                        .add(JScrollPane_ResolvedIPAddressList))));
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
            .add(jPanel3Layout.createSequentialGroup()
                .add(jPanel3Layout.createParallelGroup()
                    .add(jPanel3Layout.createSequentialGroup()
                        .add(jLabel_ResolvedIPAddressList)
                        .addContainerGap(20, 20)
                        .add(jButton_Resolve)
                        .addContainerGap(20, 20)
                        .add(jButton_Done)) 
                    .add(jPanel3Layout.createSequentialGroup()
                        .add(JScrollPane_ResolvedIPAddressList)))));

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        getContentPane().setPreferredSize(new Dimension(380, 450));
         
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
            .add(layout.createSequentialGroup()
                .add(layout.createParallelGroup()            
                    .add(jPanel_LocalDetails)
                    .add(jPanel_DomainName)
                    .add(jPanel_ResolvedDetails))));
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
            .add(layout.createSequentialGroup()
                .add(jPanel_LocalDetails)
                .addContainerGap(20, 20)
                .add(jPanel_DomainName)
                .addContainerGap(20, 20)
                .add(jPanel_ResolvedDetails)));

        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        setTitle("DNS Resolver");
        pack();
    }                                             
}